/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001-2003 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRbTable.h,v 1.8.2.7 2003/02/20 16:43:15 lyle Exp $
 ***********************************************************************/

#ifndef FXRBTABLE_H
#define FXRBTABLE_H

#define DECLARE_FXTABLEITEM_STUBS(klass,subklass) \
inline void klass ## _draw(const klass* self,const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  dynamic_cast<const subklass*>(self)->public_draw(table,dc,x,y,w,h); \
  } \
inline void klass ## _drawButton(const klass* self,const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  dynamic_cast<const subklass*>(self)->public_drawButton(table,dc,x,y,w,h); \
  } \
inline void klass ## _drawBorders(const klass* self,const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  dynamic_cast<const subklass*>(self)->public_drawBorders(table,dc,x,y,w,h); \
  } \
inline void klass ## _drawContent(const klass* self,const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  dynamic_cast<const subklass*>(self)->public_drawContent(table,dc,x,y,w,h); \
  } \
inline void klass ## _drawPattern(const klass* self,const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  dynamic_cast<const subklass*>(self)->public_drawPattern(table,dc,x,y,w,h); \
  } \
inline void klass ## _drawBackground(const klass* self,const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  dynamic_cast<const subklass*>(self)->public_drawBackground(table,dc,x,y,w,h); \
  } \
inline void klass ## _create(klass* self){ \
  self->klass::create(); \
  } \
inline void klass ## _detach(klass* self){ \
  self->klass::detach(); \
  } \
inline void klass ## _destroy(klass* self){ \
  self->klass::destroy(); \
  } \
inline void klass ## _setText(klass* self,const FXString& text){ \
  self->klass::setText(text); \
  } \
inline void klass ## _setIcon(klass* self,FXIcon* icn){ \
  self->klass::setIcon(icn); \
  } \
inline void klass ## _setFocus(klass* self,FXbool focus){ \
  self->klass::setFocus(focus); \
  } \
inline void klass ## _setSelected(klass* self,FXbool selected){ \
  self->klass::setSelected(selected); \
  } \
inline void klass ## _setEnabled(klass* self,FXbool enabled){ \
  self->klass::setEnabled(enabled); \
  } \
inline void klass ## _setDraggable(klass* self,FXbool draggable){ \
  self->klass::setDraggable(draggable); \
  } \
inline void klass ## _setIconOwned(klass* self,FXuint owned){ \
  self->klass::setIconOwned(owned); \
  } \
inline FXint klass ## _getWidth(const klass* self,const FXTable* table){ \
  return self->klass::getWidth(table); \
  } \
inline FXint klass ## _getHeight(const klass* self,const FXTable* table){ \
  return self->klass::getHeight(table); \
  }


#define IMPLEMENT_FXTABLEITEM_STUBS(klass,superklass) \
  void klass::draw(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    FXRbCallVoidMethod(this,rb_intern("draw"),table,dc,x,y,w,h); \
    } \
  void klass::public_draw(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    superklass::draw(table,dc,x,y,w,h); \
    } \
  void klass::drawButton(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    FXRbCallVoidMethod(this,rb_intern("drawButton"),table,dc,x,y,w,h); \
    } \
  void klass::public_drawButton(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    superklass::drawButton(table,dc,x,y,w,h); \
    } \
  void klass::drawBorders(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    FXRbCallVoidMethod(this,rb_intern("drawBorders"),table,dc,x,y,w,h); \
    } \
  void klass::public_drawBorders(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    superklass::drawBorders(table,dc,x,y,w,h); \
    } \
  void klass::drawContent(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    FXRbCallVoidMethod(this,rb_intern("drawContent"),table,dc,x,y,w,h); \
    } \
  void klass::public_drawContent(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    superklass::drawContent(table,dc,x,y,w,h); \
    } \
  void klass::drawPattern(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    FXRbCallVoidMethod(this,rb_intern("drawPattern"),table,dc,x,y,w,h); \
    } \
  void klass::public_drawPattern(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    superklass::drawPattern(table,dc,x,y,w,h); \
    } \
  void klass::drawBackground(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    FXRbCallVoidMethod(this,rb_intern("drawBackground"),table,dc,x,y,w,h); \
    } \
  void klass::public_drawBackground(const FXTable* table,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const { \
    superklass::drawBackground(table,dc,x,y,w,h); \
    } \
  void klass::setText(const FXString& txt){ \
    FXRbCallVoidMethod(this,rb_intern("setText"),txt); \
    } \
  void klass::setIcon(FXIcon* icn){ \
    FXRbCallVoidMethod(this,rb_intern("setIcon"),icn); \
    } \
  void klass::setFocus(FXbool focus){ \
    FXRbCallVoidMethod(this,rb_intern("setFocus"),focus); \
    } \
  void klass::setSelected(FXbool selected){ \
    FXRbCallVoidMethod(this,rb_intern("setSelected"),selected); \
    } \
  void klass::setEnabled(FXbool enabled){ \
    FXRbCallVoidMethod(this,rb_intern("setEnabled"),enabled); \
    } \
  void klass::setDraggable(FXbool draggable){ \
    FXRbCallVoidMethod(this,rb_intern("setDraggable"),draggable); \
    } \
  void klass::setIconOwned(FXuint owned){ \
    FXRbCallVoidMethod(this,rb_intern("setIconOwned"),owned); \
    } \
  FXint klass::getWidth(const FXTable* table) const { \
    return FXRbCallIntMethod(this,rb_intern("getWidth"),table); \
    } \
  FXint klass::getHeight(const FXTable* table) const { \
    return FXRbCallIntMethod(this,rb_intern("getHeight"),table); \
    } \
  void klass::create(){ \
    FXRbCallVoidMethod(this,rb_intern("create")); \
    } \
  void klass::detach(){ \
    FXRbCallVoidMethod(this,rb_intern("detach")); \
    } \
  void klass::destroy(){ \
    FXRbCallVoidMethod(this,rb_intern("destroy")); \
    }


class FXRbTableItem : public FXTableItem {
  FXDECLARE(FXRbTableItem)
protected:
  FXRbTableItem(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbTableItemVirtuals.h"
public:
  // Is this table item owned by an FXTable yet?
  FXbool owned;
public:
  // Constructor
  FXRbTableItem(const FXString& text,FXIcon* ic=NULL,void* ptr=NULL) : FXTableItem(text,ic,ptr),owned(FALSE){}

  // Mark dependencies for the GC
  static void markfunc(FXTableItem* self);

  // Destroy this object if it's not owned
  static void freefunc(FXTableItem* self);

  // Destructor
  virtual ~FXRbTableItem(){
    FXRbUnregisterRubyObj(this);
    }
  };


#define DECLARE_FXTABLE_STUBS(klass,subklass) \
inline void klass ## _drawCell(klass* self,FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint sr,FXint er,FXint sc,FXint ec){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  dynamic_cast<subklass*>(self)->public_drawCell(dc,xlo,xhi,ylo,yhi,xoff,yoff,sr,er,sc,ec); \
  } \
inline void klass ## _drawRange(klass* self,FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint rlo,FXint rhi,FXint clo,FXint chi){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  dynamic_cast<subklass*>(self)->public_drawRange(dc,xlo,xhi,ylo,yhi,xoff,yoff,rlo,rhi,clo,chi); \
  } \
inline FXTableItem* klass ## _createItem(klass* self,const FXString& text,FXIcon* icon,void* ptr){ \
  FXASSERT(self->isMemberOf(FXMETACLASS(subklass))); \
  return dynamic_cast<subklass*>(self)->public_createItem(text,icon,ptr); \
  } \
inline void klass ## _setLeadingRows(klass* self,FXint leadrows){ \
  self->klass::setLeadingRows(leadrows); \
  } \
inline void klass ## _setLeadingCols(klass* self,FXint leadcols){ \
  self->klass::setLeadingCols(leadcols); \
  } \
inline void klass ## _setTrailingRows(klass* self,FXint trailrows){ \
  self->klass::setTrailingRows(trailrows); \
  } \
inline void klass ## _setTrailingCols(klass* self,FXint trailcols){ \
  self->klass::setTrailingCols(trailcols); \
  } \
inline void klass ## _insertRows(klass* self,FXint row,FXint nr=1,FXbool notify=FALSE){ \
  self->klass::insertRows(row,nr,notify); \
  } \
inline void klass ## _insertColumns(klass* self,FXint col,FXint nc=1,FXbool notify=FALSE){ \
  self->klass::insertColumns(col,nc,notify); \
  } \
inline void klass ## _removeRows(klass* self,FXint row,FXint nr=1,FXbool notify=FALSE){ \
  FXObjectListOf<FXTableItem> items; \
  for(FXint r=row;r<row+nr;r++){ \
    for(FXint c=0;c<self->getNumCols();c++){ \
      items.append(self->getItem(r,c)); \
      } \
    } \
  self->klass::removeRows(row,nr,notify); \
  for(FXint i=0;i<items.no();i++){ \
    FXRbUnregisterRubyObj(items[i]); \
    } \
  } \
inline void klass ## _removeColumns(klass* self,FXint col,FXint nc=1,FXbool notify=FALSE){ \
  FXObjectListOf<FXTableItem> items; \
  for(FXint c=col;c<col+nc;c++){ \
    for(FXint r=0;r<self->getNumRows();r++){ \
      items.append(self->getItem(r,c)); \
      } \
    } \
  self->klass::removeColumns(col,nc,notify); \
  for(FXint i=0;i<items.no();i++){ \
    FXRbUnregisterRubyObj(items[i]); \
    } \
  } \
inline void klass ## _removeItem(klass* self,FXint row,FXint col,FXbool notify=FALSE){ \
  FXTableItem* item=self->getItem(row,col); \
  self->klass::removeItem(row,col,notify); \
  FXRbUnregisterRubyObj(item); \
  }


#define IMPLEMENT_FXTABLE_STUBS(klass,superklass) \
  void klass::drawCell(FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint sr,FXint er,FXint sc,FXint ec){ \
    FXRbCallVoidMethod(this,rb_intern("drawCell"),dc,xlo,xhi,ylo,yhi,xoff,yoff,sr,er,sc,ec); \
    } \
  void klass::public_drawCell(FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint sr,FXint er,FXint sc,FXint ec){ \
    superklass::drawCell(dc,xlo,xhi,ylo,yhi,xoff,yoff,sr,er,sc,ec); \
    } \
  void klass::drawRange(FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint rlo,FXint rhi,FXint clo,FXint chi){ \
    FXRbCallVoidMethod(this,rb_intern("drawRange"),dc,xlo,xhi,ylo,yhi,xoff,yoff,rlo,rhi,clo,chi); \
    } \
  void klass::public_drawRange(FXDC& dc,FXint xlo,FXint xhi,FXint ylo,FXint yhi,FXint xoff,FXint yoff,FXint rlo,FXint rhi,FXint clo,FXint chi){ \
    superklass::drawRange(dc,xlo,xhi,ylo,yhi,xoff,yoff,rlo,rhi,clo,chi); \
    } \
  FXTableItem* klass::createItem(const FXString& text,FXIcon* icon,void* ptr){ \
    return FXRbCallTableItemMethod(this,rb_intern("createItem"),text,icon,ptr); \
    } \
  FXTableItem* klass::public_createItem(const FXString& text,FXIcon* icon,void* ptr){ \
    return superklass::createItem(text,icon,ptr); \
    } \
  void klass::setLeadingRows(FXint leadrows){ \
    FXRbCallVoidMethod(this,rb_intern("setLeadingRows"),leadrows); \
    } \
  void klass::setLeadingCols(FXint leadcols){ \
    FXRbCallVoidMethod(this,rb_intern("setLeadingCols"),leadcols); \
    } \
  void klass::setTrailingRows(FXint trailrows){ \
    FXRbCallVoidMethod(this,rb_intern("setTrailingRows"),trailrows); \
    } \
  void klass::setTrailingCols(FXint trailcols){ \
    FXRbCallVoidMethod(this,rb_intern("setTrailingCols"),trailcols); \
    } \
  void klass::insertRows(FXint row,FXint nr,FXbool notify){ \
    FXRbCallVoidMethod(this,rb_intern("insertRows"),row,nr,notify); \
    } \
  void klass::insertColumns(FXint col,FXint nc,FXbool notify){ \
    FXRbCallVoidMethod(this,rb_intern("insertColumns"),col,nc,notify); \
    } \
  void klass::removeRows(FXint row,FXint nr,FXbool notify){ \
    FXRbCallVoidMethod(this,rb_intern("removeRows"),row,nr,notify); \
    } \
  void klass::removeColumns(FXint col,FXint nc,FXbool notify){ \
    FXRbCallVoidMethod(this,rb_intern("removeColumns"),col,nc,notify); \
    } \
  void klass::removeItem(FXint row,FXint col,FXbool notify){ \
    FXRbCallVoidMethod(this,rb_intern("removeItem"),row,col,notify); \
    }


class FXRbTable : public FXTable {
  FXDECLARE(FXRbTable)
protected:
  FXRbTable(){}
#include "FXRbObjectVirtuals.h"
#include "FXRbIdVirtuals.h"
#include "FXRbDrawableVirtuals.h"
#include "FXRbWindowVirtuals.h"
#include "FXRbScrollAreaVirtuals.h"
#include "FXRbTableVirtuals.h"
public:
  /// Make new table with nr visible rows and nc visible columns; the table
  /// is initially empty, i.e. contains no cells (nrows=0, ncols=0)
  FXRbTable(FXComposite *p,FXint nr,FXint nc,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_MARGIN,FXint pr=DEFAULT_MARGIN,FXint pt=DEFAULT_MARGIN,FXint pb=DEFAULT_MARGIN) : FXTable(p,nr,nc,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb){}

  // Mark dependencies for the GC
  static void markfunc(FXTable* self);

  // Destructor
  virtual ~FXRbTable(){
    FXRbUnregisterRubyObj(this);
    }
  };

#endif
