module Fox
  #
  # The accelerator table sends a message to a specific
  # target object when the indicated key and modifier combination
  # is pressed.
  #
  class FXAccelTable < FXObject
    #
    # Construct empty accelerator table.
    #
    def initialize # :yields: acceleratorTable
    end

    #
    # Add an accelerator to the table. The _hotKey_ is a code returned
    # by the Fox.fxparseaccel method. For example, to associate the
    # Ctrl+S keypress with sending a "save" command to a document, you
    # might use code like this:
    #
    #   hotKey = fxparseaccel("Ctrl+S")
    #   accelTable.addAccel(hotKey, doc, MKUINT(MyDocument::ID_SAVE, SEL_COMMAND))
    #
    # ==== Parameters:
    #
    # +hotKey+::	the hotkey associated with this accelerator [Integer]
    # +tgt+::		message target [FXObject]
    # +seldn+::		selector for the +SEL_KEYPRESS+ event [Integer]
    # +selup+::		selector for the +SEL_KEYRELEASE+ event [Integer]
    #
    def addAccel(hotKey, tgt=nil, seldn=0, selup=0) ; end

    #
    # Return +true+ if accelerator specified.
    # Here, _hotKey_ is a code representing an accelerator key as returned
    # by the Fox.fxparseaccel method. For example,
    #
    #   if accelTable.hasAccel?(fxparseaccel("Ctrl+S"))
    #     ...
    #   end
    #
    def hasAccel?(hotKey) ; end

    #
    # Return the target object of the given accelerator, or +nil+ if
    # the accelerator is not present in this accelerator table.
    # Here, _hotKey_ is a code representing an accelerator key as returned
    # by the Fox.fxparseaccel method. For example,
    #
    #   doc = accelTable.targetofAccel(fxparseaccel("Ctrl+S"))
    #
    def targetOfAccel(hotKey) ; end
  
    #
    # Remove mapping for specified hot key.
    # Here, _hotKey_ is a code representing an accelerator key as returned
    # by the Fox.fxparseaccel method. For example,
    #
    #   accelTable.removeAccel(fxparseaccel("Ctrl+S"))
    #
    def removeAccel(hotKey) ; end
  end
end
