/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001-2003 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/


/// CUR Cursor class
class FXCURCursor : public FXCursor {
protected:
  FXCURCursor(){}
public:
  %extend {
    /// Construct CUR cursor from memory stream in Microsoft CUR format
    FXCURCursor(FXApp* a,const void* pix){
      return new FXRbCURCursor(a,pix);
      }
    }

  /// Destructor
  virtual ~FXCURCursor();
  };


DECLARE_FXOBJECT_VIRTUALS(FXCURCursor)
DECLARE_FXID_VIRTUALS(FXCURCursor)
DECLARE_FXCURSOR_VIRTUALS(FXCURCursor)

%{
/* Copied from fxcurio.cpp */
#define DWORD_ALIGN_BITS(b)   (((b) / 32) + (((b) % 32 > 0) ? 1 : 0))
%}

%inline %{
/// Load a CUR file from a stream
VALUE fxloadCUR(FXStream& store){
  FXuchar* source;
  FXuchar* mask;
  FXint width;
  FXint height;
  FXint xspot;
  FXint yspot;
  FXint bytesPerMaskLine;
  FXint maskLength;
  
  if(fxloadCUR(store,source,mask,width,height,xspot,yspot)){
    bytesPerMaskLine=4*DWORD_ALIGN_BITS(height);
    maskLength=bytesPerMaskLine*height;
    VALUE ary=rb_ary_new();
    rb_ary_push(ary,rb_str_new(reinterpret_cast<char*>(source),maskLength));
    rb_ary_push(ary,rb_str_new(reinterpret_cast<char*>(mask),maskLength));
    rb_ary_push(ary,to_ruby(width));
    rb_ary_push(ary,to_ruby(height));
    rb_ary_push(ary,to_ruby(xspot));
    rb_ary_push(ary,to_ruby(yspot));
    FXFREE(&source);
    FXFREE(&mask);
    return ary;
    }
  else{
    return Qnil;
    }
  }
%}

