/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001-2003 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/


%apply const FXuchar *PIXELS { const FXuchar *data };

/// GIF Cursor class
class FXGIFCursor : public FXCursor {
public:
  %extend {
    /// Construct a cursor from memory stream formatted as CompuServe GIF format
    FXGIFCursor(FXApp* a,const void* pix,FXint hx=-1,FXint hy=-1){
      return new FXRbGIFCursor(a,pix,hx,hy);
      }
    }

  /// Destroy
  virtual ~FXGIFCursor(){}
  };

DECLARE_FXOBJECT_VIRTUALS(FXGIFCursor)
DECLARE_FXID_VIRTUALS(FXGIFCursor)
DECLARE_FXCURSOR_VIRTUALS(FXGIFCursor)


/// Save a gif file to a stream
FXbool fxsaveGIF(FXStream& store,const FXuchar *data,FXColor transp,FXint width,FXint height);


%inline %{
  /// Load a GIF file from a stream
  VALUE fxloadGIF(FXStream& store){
    FXuchar* data;
    FXColor transp;
    FXint width;
    FXint height;
    if(fxloadGIF(store,data,transp,width,height)){
      VALUE ary=rb_ary_new();
      rb_ary_push(ary,rb_str_new((const char*)data,3*width*height));
      FXFREE(&data);
      rb_ary_push(ary,to_ruby(transp));
      rb_ary_push(ary,to_ruby(width));
      rb_ary_push(ary,to_ruby(height));
      return ary;
      }
    else{
      return Qnil;
      }
    }
%}
